#!/bin/sh

log="/var/log/senseshield_install.log"
SENSEPATH=/usr/lib/senseshield/senseshield

export PATH="$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
 
write_log()
{	
    echo "[$0:"`date "+%Y-%m-%d %X %s"`"]" $1  >> $log
}

if grep -Eqi "CentOS" /etc/issue || grep -Eq "CentOS" /etc/*-release; then
    LINUXNAME='CentOS'
elif grep -Eqi "Ubuntu" /etc/issue || grep -Eq "Ubuntu" /etc/*-release; then
    LINUXNAME='Ubuntu'
elif grep -Eqi "Debian" /etc/issue || grep -Eq "Debian" /etc/*-release; then
    LINUXNAME='Debian'
elif grep -Eqi "Fedora" /etc/issue || grep -Eq "Fedora" /etc/*-release; then
    LINUXNAME='Fedora'
else
    LINUXNAME='UnKown'
fi


install()
{
    cd /tmp
    
    if [ "$1" = "DEB" ] ; then
        if [ $(getconf WORD_BIT) = '32' ] && [ $(getconf LONG_BIT) = '64' ] ; then
            if [ ! -f "x64/senseshield" ] ; then
                echo "********************************************************"
                echo "install failed. this package only support 32 bit Linux OS."
                echo "********************************************************"
                exit -1
            fi
        else
            if [ ! -f "x86/senseshield" ] ; then
                echo "********************************************************"
                echo "install failed. this package only support 64 bit Linux OS."
                echo "********************************************************"
                exit -1
            fi
        fi
	fi
	if [ "$1" = "TAR" ] ; then
        cd ./senseshield_pkg_file/senseshield_pkg
    fi
	# 2. 创建文件夹
	mkdir -p /usr/lib/senseshield/etc/ss >> $log
	mkdir -p /usr/lib/senseshield/lib >> $log
	mkdir -p /lib/systemd/system  >> $log
	
    
	version=`cat config/version | grep -i "version" | awk -F ":" '{print $2}'`
	write_log "Version:"$version
    
	cp config/version /usr/lib/senseshield/version
	cp config/sssynccfg.ini /usr/lib/senseshield/sssynccfg.ini
	if [ ! -f "/usr/lib/senseshield/ss_config.xml" ];then
		cp config/ss_config.xml /usr/lib/senseshield/ss_config.xml
	fi
	if [ ! -f "/usr/lib/senseshield/etc/ss/authconfig.xml" ];then
		cp config/authconfig.xml /usr/lib/senseshield/etc/ss/authconfig.xml
	fi
	cp config/extra.sh /usr/lib/senseshield/extra.sh
    if [ "$1" = "TAR" ] ; then
        cp config/senseshieldd  /etc/init.d/senseshieldd
    else
        if [ -f "/bin/systemctl" ] ; then
            cp config/senseshield.service  /lib/systemd/system/senseshield.service
        else
            cp config/senseshieldd  /etc/init.d/senseshieldd
        fi
    fi
    
	if [ $(getconf WORD_BIT) = '32' ] && [ $(getconf LONG_BIT) = '64' ] ; then
		cp -f x64/senseshield /usr/lib/senseshield/senseshield
		cp -f x64/sssync /usr/lib/senseshield/sssync
		cp -f x64/ssclt /usr/bin/ssclt
		cp -f x64/sscfg /usr/bin/sscfg
		cp -f x64/libssl.so.1.0.0 /usr/lib/senseshield/lib/libssl.so.1.0.0
		cp -f x64/libcrypto.so.1.0.0 /usr/lib/senseshield/lib/libcrypto.so.1.0.0
		cp -f x64/liblib_net_agent_shared.so /usr/lib/senseshield/lib/liblib_net_agent_shared.so
	else
		cp -f x86/senseshield /usr/lib/senseshield/senseshield
		cp -f x86/sssync /usr/lib/senseshield/sssync
		cp -f x86/ssclt /usr/bin/ssclt
		cp -f x86/sscfg /usr/bin/sscfg
		cp -f x86/libssl.so.1.0.0 /usr/lib/senseshield/lib/libssl.so.1.0.0
		cp -f x86/libcrypto.so.1.0.0 /usr/lib/senseshield/lib/libcrypto.so.1.0.0
		cp -f x86/liblib_net_agent_shared.so /usr/lib/senseshield/lib/liblib_net_agent_shared.so
	fi
		

	# 4. 添加权限
	write_log "add permission "
	chmod 755 /usr/lib/senseshield/version
	chmod 755 /usr/lib/senseshield/sssynccfg.ini
	chmod 755 /usr/lib/senseshield/ss_config.xml
	chmod 755 /usr/lib/senseshield/etc/ss/authconfig.xml
	chmod 755 /usr/lib/senseshield/extra.sh
	
	chmod 744 /usr/lib/senseshield/senseshield
	chmod 744 /usr/lib/senseshield/sssync
	chmod 755 /usr/bin/ssclt
	chmod 755 /usr/bin/sscfg
	chmod 755 /usr/lib/senseshield/lib/libssl.so.1.0.0
	chmod 755 /usr/lib/senseshield/lib/libcrypto.so.1.0.0
	chmod 755 /usr/lib/senseshield/lib/liblib_net_agent_shared.so
		
	
	# 5. 添加服务
	write_log "add service"
	#deb安装包会将脚本所在的目录文件全部安装包到系统里 我们的服务管理根据高低版本支持会有两套解决方法
	#所以需要互相删除不需要的配置文件，不删除则会产生服务命令不起作用的问题

    if [ "$1" = "TAR" ] ; then
        rm -f /lib/systemd/system/senseshield.service
		chmod 755 /etc/init.d/senseshieldd
		cd /etc/rc2.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
		cd /etc/rc3.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
		cd /etc/rc4.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
		cd /etc/rc5.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
		/etc/init.d/senseshieldd start
    else
        if [ -f "/bin/systemctl" ] ; then
            echo "daemon-reload " >> $log
            rm -f /etc/init.d/senseshieldd
            chmod 644 /lib/systemd/system/senseshield.service
            cd /lib/systemd/system
            systemctl daemon-reload
            echo `systemctl enable senseshield.service 2>&1` >> $log
            systemctl start senseshield.service
        else
            rm -f /lib/systemd/system/senseshield.service
            chmod 755 /etc/init.d/senseshieldd
            cd /etc/rc2.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
            cd /etc/rc3.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
            cd /etc/rc4.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
            cd /etc/rc5.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
            /etc/init.d/senseshieldd start
        fi
    fi
	

	#6. 启动同步进程
	/usr/lib/senseshield/sssync
	if [ -f "/etc/rc.d/rc.local" ] ; then
		sed -i '/sssync/d' /etc/rc.d/rc.local
		line_number=`grep -nv "#" /etc/rc.d/rc.local | head -1 | awk -F ":" '{print $1}'`
		if [ x$line_number = x ] ; then
			line_number="1"
			echo "" > /etc/rc.d/rc.local
		fi
		sed -i "${line_number}i\/usr/lib/senseshield/sssync" /etc/rc.d/rc.local
	fi
	if [ ! -f "/etc/rc.local" ] ; then
		touch /etc/rc.local
		echo "#!/bin/sh" >> /etc/rc.local
		echo "/usr/lib/senseshield/sssync" >> /etc/rc.local
		echo "exit 0" >> /etc/rc.local
		chmod 755 /etc/rc.local
		if [ -f "/etc/rc.d/rc.local" ] ; then
			chmod 755 /etc/rc.d/rc.local
		fi
	else
        sed -i '/sssync/d' /etc/rc.local
		line_number=`grep -nv "#" /etc/rc.local | head -1 | awk -F ":" '{print $1}'`
		if [ x$line_number = x ] ; then
		line_number="1"
		echo "" > /etc/rc.local
		fi
		sed -i "${line_number}i\/usr/lib/senseshield/sssync" /etc/rc.local
		chmod +x /etc/rc.local
		if [ -f "/etc/rc.d/rc.local" ] ; then
			chmod +x /etc/rc.d/rc.local
		fi
	fi
	ret=`ps -ef|grep "$SENSEPATH" |grep -v "grep"`
	if [ x"$ret" != x ] ; then
		echo "install success."
	else 
		echo "install failed. senseshield service is not running."
	fi
    if [ "$1" = "RPM" ] ; then
        rm -rf x86/ x64/ config/
    fi
	write_log "install end"

}

uninstall()
{
	write_log "uninstall"
	version=`cat /usr/lib/senseshield/version | grep -i "version" | awk -F ":" '{print $2}'`
	write_log "Version:"$version

	write_log "kill sssync"
	ret=`ps -ef|grep sssync |grep -v "grep"`
	if [ x"$ret" != x ]
	then
		kill -9 $(pidof sssync)
	fi

	sed -i '/sssync/d' /etc/rc.local
	if [ -f "/etc/rc.d/rc.local" ] ; then
		sed -i '/sssync/d' /etc/rc.d/rc.local
	fi
	write_log "remove service"
    
    if [ "$1" = "TAR" ] ; then
		/etc/init.d/senseshieldd stop
		rm -f /etc/init.d/senseshieldd
		rm -f /etc/rc2.d/S99senseshieldd
		rm -f /etc/rc3.d/S99senseshieldd
		rm -f /etc/rc4.d/S99senseshieldd
		rm -f /etc/rc5.d/S99senseshieldd
    else
        if [ -f "/bin/systemctl" ] ; then
            cd /lib/systemd/system
            systemctl stop senseshield.service
            echo  `systemctl disable senseshield.service 2>&1`  >> $log
            rm -f /lib/systemd/system/senseshield.service >> $log
        else
            /etc/init.d/senseshieldd stop
            rm -f /etc/init.d/senseshieldd
            rm -f /etc/rc2.d/S99senseshieldd
            rm -f /etc/rc3.d/S99senseshieldd
            rm -f /etc/rc4.d/S99senseshieldd
            rm -f /etc/rc5.d/S99senseshieldd
        fi
    fi
    
	write_log "delete files"
	rm -f /usr/lib/senseshield/senseshield
	rm -f /usr/lib/senseshield/sssync
	rm -f /usr/lib/senseshield/ss_config.xml
	rm -f /usr/lib/senseshield/sssynccfg.ini    
    rm -f /usr/lib/senseshield/extra.sh	
	rm -f /usr/lib/senseshield/version
    rm -f /usr/bin/ssclt
	rm -f /usr/bin/sscfg
	rm -rf /usr/lib/senseshield/lib

	echo "uninstall success"
}

iptables_add_port()
{
    if [ "$1" = "TAR" ] ; then
        /sbin/iptables -I INPUT -p tcp --dport 10334  -j ACCEPT >> $log
        /sbin/iptables -I INPUT -p udp --dport 10335  -j ACCEPT >> $log
        /sbin/iptables -I INPUT -p udp --dport 10336  -j ACCEPT >> $log
    else
        if [ -f "/usr/bin/firewall-cmd" -a $LINUXNAME != "Fedora" ];then
            firewall-cmd --zone=public --add-port=10334/tcp  >> $log
            firewall-cmd --zone=public --add-port=10335/udp  >> $log
            firewall-cmd --zone=public --add-port=10336/udp  >> $log
        elif [ -f "/usr/sbin/ufw" ];then
            ufw allow 10334/tcp >> $log
            ufw allow 10335/udp >> $log
            ufw allow 10336/udp >> $log	
        else
            /sbin/iptables -I INPUT -p tcp --dport 10334  -j ACCEPT >> $log
            /sbin/iptables -I INPUT -p udp --dport 10335  -j ACCEPT >> $log
            /sbin/iptables -I INPUT -p udp --dport 10336  -j ACCEPT >> $log
        fi
    fi	
}

iptables_del_port()
{
    if [ "$1" = "TAR" ] ; then
        /sbin/iptables -D INPUT -p tcp --dport 10334  -j ACCEPT >> $log
        /sbin/iptables -D INPUT -p udp --dport 10335  -j ACCEPT >> $log
        /sbin/iptables -D INPUT -p udp --dport 10336  -j ACCEPT >> $log
    else
        if [ -f "/usr/bin/firewall-cmd"  -a $LINUXNAME != "Fedora" ];then
            firewall-cmd --zone=public --remove-port=10334/tcp  >> $log
            firewall-cmd --zone=public --remove-port=10335/udp  >> $log
            firewall-cmd --zone=public --remove-port=10336/udp  >> $log
        elif [ -f "/usr/sbin/ufw" ];then
            ufw delete allow 10334/tcp >> $log
            ufw delete allow 10335/udp >> $log
            ufw delete allow 10336/udp >> $log	
        else
            /sbin/iptables -D INPUT -p tcp --dport 10334  -j ACCEPT >> $log
            /sbin/iptables -D INPUT -p udp --dport 10335  -j ACCEPT >> $log
            /sbin/iptables -D INPUT -p udp --dport 10336  -j ACCEPT >> $log
        fi
    fi
}

if [ "$1" = "install" ] ; then
    install $2
elif [ "$1" = "uninstall" ] ; then
	uninstall $2
elif [ "$1" = "addport" ] ; then
	iptables_add_port 
elif [ "$1" = "delport" ] ; then
	iptables_del_port 
fi

exit 0
